#ifndef _QUADROTOR_H_
#define _QUADROTOR_H_

#include "../types.h"

#include<vector>
typedef std::vector<double> Distance;

namespace imr {
	namespace dynamics {
		class Quadrotor
		{
		private:
			/// Gravitation vector
			Vector g;

			/// Calculation of the new rotation matrices
			const Matrix calculate_Rw(const Vector & omega);

			/// Moves the quadrotor with given control inputs (duration DELTA_T)
			void move(const int time, const double f, const Vector moments);

			Vector get_euler_angles(const Matrix & R);

		protected:
			/// Type of the robot
			// 0 - leader (the control is only stored for the leader)
			const int type;
			double traveled_fix;
			Distance traveled_distance;
			Distance traveled_distance_fdif;

			/// Quadrotor properties
			// Matrix of inertia
			Matrix J;
			Matrix invJ;

			// Weight
			double m;

			/// Actual state
			State state;

			/// Total flight time
			double total_time;

			/// Simulation history
			StateHistory state_history;			
			StateHistory state_history_fdif;
			ControlHistory control_history;

			/// Moves the quadrotor with given control inputs (duration t)
			void static_control_move(const double t, const double f, const Vector moments);
			Quadrotor() : type(-1) { };

		public:
			void setRot(const double angle, const Vector formationCenter, double diameter);
			void setRotFol(const double angle);
			Quadrotor(const Vector & initial_X, const Vector & initial_V, const Vector & initial_OMEGA,
																	const Matrix & initial_R, const int _type);
		};
	}
}
#endif
