/**
 * main.cpp
 * 
 * The main file.
 * 
 * @author Zdenek Kasl (kaslzden@fel.cvut.cz)
 */

#define DEFINE_WORLD

#include <iostream>
#include <Eigen/Core>

#include "types.h"
#include "world.h"

#include "map/mapLoader3d.h"
#include "map/map.h"

typedef imr::mapLoader3d::MapLoader3d Loader;
typedef imr::map::Map Map;

#include "robot/robot_leader.h"
#include "robot/robot_follower.h"

typedef imr::robot::RobotLeader Leader;
typedef imr::robot::RobotFollower Follower;

/**
 * The main function
 * */
int main(int argc, char **argv)
{
    if (SAMPLE_T < DELTA_T) {
        std::cerr << "settings.h: Sampling period is smaller than dela_t!" << std::endl;
        exit(EXIT_FAILURE);
    }

    Vector start(0.0, 0.0, 0.0);
    Vector goal(3.0, 3.0, 0.0);

    /// Create a map instance and add it to structure world
   Loader loader(MAP_FILE);
	
    world.map = new Map(start, goal, loader);

    Matrix R0;
    Vector X0, V0, W0;
	R0 << 1,0,0,
	      0,1,0,
	      0,0,1;

	FSpec fs(0.5, 0.4, 0.0);

	int id = 1;
	int n = 10;
	double defect_time = 1.0; // [s]

	X0 << -fs.p,fs.q,fs.r;
	V0 << 0,0,0;
	W0 << 0,0,0;

	std::cout << "Follower" << id << " - initial rotation matrix:\n" << R0 << std::endl;
	std::cout << "Follower" << id << " - initial position:\n(" << X0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial velocity:\n(" << V0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial angular velocity:\n(" << W0.transpose() << ")'" << std::endl;

	world.followers.push_back(new Follower(id, fs, n, defect_time, imr::defects::FolDefNone, X0, V0, W0, R0, "../scripts/fol1Loader.m"));
	id ++;
	fs = FSpec(0.5, -0.4, 0.0);

	X0 << -fs.p,fs.q,fs.r;
	V0 << 0,0,0;
	W0 << 0,0,0;

	std::cout << "Follower" << id << " - initial rotation matrix:\n" << R0 << std::endl;
	std::cout << "Follower" << id << " - initial position:\n(" << X0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial velocity:\n(" << V0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial angular velocity:\n(" << W0.transpose() << ")'" << std::endl;

	world.followers.push_back(new Follower(id, fs, n, defect_time, imr::defects::FolDefNone, X0, V0, W0, R0, "../scripts/fol2Loader.m"));

	/*
id ++;
	fs = FSpec(0.9, 0.4, 0.0);

	X0 << -fs.p,fs.q,fs.r;
	V0 << 0,0,0;
	W0 << 0,0,0;

	std::cout << "Follower" << id << " - initial rotation matrix:\n" << R0 << std::endl;
	std::cout << "Follower" << id << " - initial position:\n(" << X0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial velocity:\n(" << V0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial angular velocity:\n(" << W0.transpose() << ")'" << std::endl;

	world.followers.push_back(new Follower(id, fs, n, defect_time, imr::defects::FolDefNone, X0, V0, W0, R0, "../scripts/fol3Loader.m"));
	
	id ++;
	fs = FSpec(0.9, -0.4,-0.0);

	X0 << -fs.p,fs.q,fs.r;
	V0 << 0,0,0;
	W0 << 0,0,0;

	std::cout << "Follower" << id << " - initial rotation matrix:\n" << R0 << std::endl;
	std::cout << "Follower" << id << " - initial position:\n(" << X0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial velocity:\n(" << V0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial angular velocity:\n(" << W0.transpose() << ")'" << std::endl;

world.followers.push_back(new Follower(id, fs, n, defect_time, imr::defects::FolDefNone, X0, V0, W0, R0, "../scripts/fol4Loader.m"));
/*	id ++;
	fs = FSpec(0.8, 0.4,-0.25);

	X0 << -fs.p,fs.q,fs.r;
	V0 << 0,0,0;
	W0 << 0,0,0;

	std::cout << "Follower" << id << " - initial rotation matrix:\n" << R0 << std::endl;
	std::cout << "Follower" << id << " - initial position:\n(" << X0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial velocity:\n(" << V0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial angular velocity:\n(" << W0.transpose() << ")'" << std::endl;

	world.followers.push_back(new Follower(id, fs, n, defect_time, imr::defects::FolDefNone, X0, V0, W0, R0, "../scripts/fol5Loader.m"));
id ++;
	fs = FSpec(0.3, 0.4,0.25);

	X0 << -fs.p,fs.q,fs.r;
	V0 << 0,0,0;
	W0 << 0,0,0;

	std::cout << "Follower" << id << " - initial rotation matrix:\n" << R0 << std::endl;
	std::cout << "Follower" << id << " - initial position:\n(" << X0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial velocity:\n(" << V0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial angular velocity:\n(" << W0.transpose() << ")'" << std::endl;

	world.followers.push_back(new Follower(id, fs, n, defect_time, imr::defects::FolDefNone, X0, V0, W0, R0, "../scripts/fol6Loader.m"));
id ++;
	fs = FSpec(0.3, -0.4,0.25);

	X0 << -fs.p,fs.q,fs.r;
	V0 << 0,0,0;
	W0 << 0,0,0;

	std::cout << "Follower" << id << " - initial rotation matrix:\n" << R0 << std::endl;
	std::cout << "Follower" << id << " - initial position:\n(" << X0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial velocity:\n(" << V0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial angular velocity:\n(" << W0.transpose() << ")'" << std::endl;

	world.followers.push_back(new Follower(id, fs, n, defect_time, imr::defects::FolDefNone, X0, V0, W0, R0, "../scripts/fol7Loader.m"));

	
id ++;
	fs = FSpec(0.55, 0.0, 0.25);

	X0 << -fs.p,fs.q,fs.r;
	V0 << 0,0,0;
	W0 << 0,0,0;

	std::cout << "Follower" << id << " - initial rotation matrix:\n" << R0 << std::endl;
	std::cout << "Follower" << id << " - initial position:\n(" << X0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial velocity:\n(" << V0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial angular velocity:\n(" << W0.transpose() << ")'" << std::endl;

	world.followers.push_back(new Follower(id, fs, n, defect_time, imr::defects::FolDefNone, X0, V0, W0, R0, "../scripts/fol8Loader.m"));
	
	id ++;
	fs = FSpec(0.8, -0.4,0.25);

	X0 << -fs.p,fs.q,fs.r;
	V0 << 0,0,0;
	W0 << 0,0,0;

	std::cout << "Follower" << id << " - initial rotation matrix:\n" << R0 << std::endl;
	std::cout << "Follower" << id << " - initial position:\n(" << X0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial velocity:\n(" << V0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial angular velocity:\n(" << W0.transpose() << ")'" << std::endl;

	world.followers.push_back(new Follower(id, fs, n, defect_time, imr::defects::FolDefNone, X0, V0, W0, R0, "../scripts/fol9Loader.m"));
id ++;
	fs = FSpec(0.8, 0.4,0.25);

	X0 << -fs.p,fs.q,fs.r;
	V0 << 0,0,0;
	W0 << 0,0,0;

	std::cout << "Follower" << id << " - initial rotation matrix:\n" << R0 << std::endl;
	std::cout << "Follower" << id << " - initial position:\n(" << X0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial velocity:\n(" << V0.transpose() << ")'" << std::endl;
	std::cout << "Follower" << id << " - initial angular velocity:\n(" << W0.transpose() << ")'" << std::endl;

	world.followers.push_back(new Follower(id, fs, n, defect_time, imr::defects::FolDefNone, X0, V0, W0, R0, "../scripts/fol10Loader.m"));
*/	X0 << 0,0,0;
	V0 << 0,0,0;
	W0 << 0,0,0;

	std::cout << "Initial leader's rotation matrix:\n" << R0 << std::endl;
	std::cout << "Initial leader's position:\n(" << X0.transpose() << ")'" << std::endl;
	std::cout << "Initial leader's velocity:\n(" << V0.transpose() << ")'" << std::endl;
	std::cout << "Initial leader's angular velocity:\n(" << W0.transpose() << ")'" << std::endl;

	world.leader = new Leader(0, 5, X0, V0, W0, R0, "../scripts/leaderLoader.m");
	world.leader->run();

    return EXIT_SUCCESS;
}
