#include "map.h"
#include "mapLoader3d.h"

#include <cmath>
#include <vector>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <iostream>

using namespace std;
using namespace imr::map;
namespace m = imr::mapLoader3d;
extern "C" {
    #include "../gjk/gjk.h"

    // Transformations definition
    double (*tr1)[DIM + 1] = 0, (*tr2)[DIM + 1] = 0;

    // Points
    double pts1[8][3];
    double pts2[8][3];
    double (*ppts1)[3];
    double (*ppts2)[3];

    // Witness points - may be used to speedup distance calculation
    double wit1[DIM], wit2[DIM];
    
    // Simplex record
    // Used for witness points calculation
    struct simplex_point simplex_record;
    
    // Apply trans method (GJK)
    void apply_trans(  Transform t, REAL * src, REAL * tgt)
    {
      int i;
    
      if ( t==0 )
        for ( i=0 ; i<DIM ; i++ )
          tgt[i] = src[i];
      else {
        for ( i=0 ; i<DIM ; i++ )
          tgt[i] = t[i][DIM] + OTHER_DOT_PRODUCT( t[i], src);
      }
      return;
    }
    
    // Apply rotation transpose method (GJK)
    void apply_rot_transpose( Transform t, REAL * src, REAL * tgt)
    {
      int i;
    
      if ( t==0 )
        for ( i=0 ; i<DIM ; i++ )
          tgt[i] = src[i];
      else {
        for ( i=0 ; i<DIM ; i++ )
          tgt[i] = DO_MULTIPLY( t[0][i], src[0]) + DO_MULTIPLY( t[1][i], src[1])
    	             + DO_MULTIPLY( t[2][i], src[2]);
      }
      return;
    }
}

Map::Map(const Vector & sp, const Vector & gp, const Loader & loader) : start(sp), goal(gp)
{
	// Load obstacles
	obstacles = loader.get_obstacles();
	// Plan initial trajectory
	// TODO initial trajectory planning
initial_trajectory.push_back(OptVec(0.0, 0.40,0.0,0.0,3.0));
initial_trajectory.push_back(OptVec(0.0, 0.40,0.0,0.0,2.0));
initial_trajectory.push_back(OptVec(0.0, 0.40,0.0,0.0,4.5));
initial_trajectory.push_back(OptVec(1.57, 0.0,0.0,0.0,0.2));
initial_trajectory.push_back(OptVec(0.0, 0.40,0.0,0.0,2.6));
initial_trajectory.push_back(OptVec(0.0, 0.40,0.0,0.0,4.4));	

	//save();
}

/**
 * Returns maximal distance to obstacles
 */
double Map::obstacle_distance(const Point & p, const double t)
{
	double tobs = 0;
	double ret_val = 10^50;
	double tmp[8][3] ={{p[0],p[1],p[2]},
			  {p[0],p[1],p[2]},
			  {p[0],p[1],p[2]},
		          {p[0],p[1],p[2]},
		 	  {p[0],p[1],p[2]},
			  {p[0],p[1],p[2]},
			  {p[0],p[1],p[2]},
			  {p[0],p[1],p[2]}};
			
			   double pts1[8][3];
 			   double pts2[8][3];
   		     	   double (*ppts1)[3];
    			   double (*ppts2)[3];
			   int num_pts = 8;
    			   double sqdist = 0.0;
    			   bool wp_use = 0;
   int rings[40] = {8,12,16,20,24,28,32,36,
                                 1,2,4,-1,
                                 0,3,5,-1,
                                 0,3,6,-1,
                                 1,2,7,-1,
                                 0,5,6,-1,
                                 1,4,7,-1,
                                 2,4,7,-1,
                                 3,5,6,-1};

    Object_structure o1, o2;

    o1.numpoints = num_pts;
    o1.vertices = tmp;
    o1.rings = &rings[0];
    o2.numpoints = num_pts;
    o2.rings = &rings[0];
 
for(int i = 0; i<obstacles.size(); i++)
{
    tobs = obstacles[i].time;
    if (tobs < t){
    o2.vertices = obstacles[i].nodes;    
    sqdist = gjk_distance(&o1, tr1,
                          &o2, tr2,
                          wit1, wit2, &simplex_record, wp_use);
	

	if (sqrt(sqdist) < ret_val) {
		ret_val = sqrt(sqdist);
	}

}
}
    



	return ret_val;
}

/**
 * Returns distance to goal from the specified position
 */
double Map::goal_distance(const Vector & v)
{
	//obst = sqr(min(0.0, (curr - so->rs)/(curr - so->ra)));
	double ret_val = dist(v,goal);
	return ret_val;
}

/**
 * Saves the map to file readable by matlab or octave
 
void Map::save()
{
	ofstream out("../scripts/loadMap.m");

    /// Obstackle iterator
	vector<Obstacle>::iterator obst_it;
	obst_it = obstacles.begin();

    out << "oX = [ ";
    while (obst_it != obstacles.end()) {
        out << obst_it->get_center()(0) << " ";
        obst_it++;
    }
    out << "];\n";

    obst_it = obstacles.begin();
    out << "oY = [ ";
    while (obst_it != obstacles.end()) {
        out << obst_it->get_center()(1) << " ";
        obst_it++;
    }
    out << "];\n";

    obst_it = obstacles.begin();
    out << "oZ = [ ";
    while (obst_it != obstacles.end()) {
        out << obst_it->get_center()(2) << " ";
        obst_it++;
    }
    out << "];\n";

    obst_it = obstacles.begin();
    out << "oR = [ ";
    while (obst_it != obstacles.end()) {
        out << obst_it->get_radius() << " ";
        obst_it++;
    }
    out << "];\n";

    out << std::flush;
    out.close();

    cout << "Matlab file 'scripts/loadMap.m' created!" << endl;
}*/
