#include "robot.h"

#include<iostream>

using namespace std;
using namespace imr::robot;

/**
 * Constructor of the class Robot.
 * @param id            - id of the robot 0 - leader, 1..n - followers
 * @param n             - number of constant duration control inputs
 * @param initial_X     - initial position of the quadcopter
 * @param initial_V     - initial velocity of the quadcopter
 * @param initial_OMEGA - initial angular velocity of the quadcopter
 * @param initial_R     - initial rotation matrix of the quadcopter
 * @param filename      - file to store the simulation results
 * */
Robot::Robot(const int _id, const int _n, const Vector & initial_X, const Vector & initial_V, const Vector & initial_OMEGA,
    	   const Matrix & initial_R, const char * filename) : time(0.0), id(_id), n(_n),
    	   	   robot_dynamics(initial_X, initial_V, initial_OMEGA, initial_R, _id, filename), limits()
{

}

/**
 * Sampling of the trajectory
 */
Samples Robot::sample_trajectory(OptVec * traj, const int len, const int nsamples, const int msamples, const int es, const char * filename)
{
	std::cerr << "Implement virtual function Robot::sample_trajectory()!" << std::endl;
	exit(EXIT_FAILURE);
}

/**
 * Movement of the robot
 */
void Robot::robot_move(const OptVec & ctr)
{
	std::cerr << "Implement virtual function Robot::robot_move()!" << std::endl;
	exit(EXIT_FAILURE);
}

/**
 * Robots control loop
 */
void Robot::run()
{
	std::cerr << "Implement virtual function Robot::run()!" << std::endl;
	exit(EXIT_FAILURE);
}

/**
 * Robots initialization
 */
void Robot::init()
{
	std::cerr << "Implement virtual function Robot::init()!" << std::endl;
	exit(EXIT_FAILURE);
}

/**
 * Returns expected state when the traveled path is equal to 's'.
 */
const State Robot::state_estimation_s(const double s)
{
	
	double s_to_go = s - traveled_s;
	State ret_val(robot_dynamics.get_state());

	double yaw_inc = 0.0;
	double t = 0.0;
	int i = 0;

	while (s_to_go > imr::constants::eps) {
		t = min(traj[i].t,s/traj[i].vt);
	if (traj[i].ag > 0.01){
		ret_val.EA(2) += traj[i].ag;
	}

        if (abs(traj[i].c) > constants::eps) {
        	yaw_inc = traj[i].c*traj[i].vt*t;
			ret_val.X(0) += (sin(ret_val.EA(2)+yaw_inc) - sin(ret_val.EA(2)))/traj[i].c;
			ret_val.X(1) -= (cos(ret_val.EA(2)+yaw_inc) - cos(ret_val.EA(2)))/traj[i].c;
			ret_val.X(2) += traj[i].vn*t;
			ret_val.EA(2) += yaw_inc;
        } else {
			ret_val.X(0) += traj[i].vt*cos(ret_val.EA(2))*t;
			ret_val.X(1) += traj[i].vt*sin(ret_val.EA(2))*t;
			ret_val.X(2) += traj[i].vn*t;
        }

		s_to_go -= t*traj[i].vt;
		i ++;
	}
	return ret_val;
}
