#ifndef ROBOT_H
#define ROBOT_H

#include <vector>

#include "../types.h"
#include "../settings.h"
#include "../dynamics/dynamics.h"

typedef imr::dynamics::Dynamics Dynamics;

namespace imr {
	namespace robot {
		class Robot
		{
		protected:
			/// Robot's dynamics
			Dynamics robot_dynamics;

			/// Total time of the simulation --> important for formation keeping
			double time;

			/// Trajectory
			const int n;
			Trajectory traj;

			/// Robot id
			const int id;

			/// Traveled distance in xy plane
			double traveled_s;
			
			/// Distance traveled in time of the last rotation			
			int rot_s;

			double ra, rs;
			Limits limits;
			FSpec fs;

			virtual void init();
			const State state_estimation_s(const double s);

		public:
			Robot(const int _id, const int _n, const Vector & initial_X, const Vector & initial_V, const Vector & initial_OMEGA,
				   const Matrix & initial_R, const char * filename);

			const State get_current_state() const { return robot_dynamics.get_state(); }
			const Limits get_limits() const { return limits; };

			const double get_ra() const { return ra; }
			const double get_rs() const { return rs; }

			const double cMax() const { return limits.cMax; }
			const double cMin() const { return limits.cMin; }

			const double vtMax() const { return limits.vtMax; }
			const double vtMin() const { return limits.vtMin; }

			virtual Samples sample_trajectory(OptVec * traj, const int len, const int nsamples, const int msamples, const int es, const char * filename);
			virtual void robot_move(const OptVec & ctr);
			virtual void run();

            double get_time() { return time; }

			virtual ~Robot() {  }
		};
	}
}
#endif
