#ifndef ROBOT_FOLLOWER_H
#define ROBOT_FOLLOWER_H

#include "robot.h"
#include "../settings.h"

#include "../measurement/obj_logger.h"
#include "../measurement/time_measurement.h"

typedef imr::logger::ObjLogger ObjLogger;
typedef imr::time::TimeMeasurement TimeMeasurement;
typedef imr::defects::FollowerDefect FollowerDefect;

using namespace std;

namespace imr {
	namespace robot {
		class RobotFollower : public Robot
		{
		private:
			FSpec fs;
			FSpec fsoriginal;			
			Trajectory leadertraj;

			const FollowerDefect defect;
			const double defect_time;
			const double fault_vt;
			const double fault_vn;
			const double fault_ag;


			void init();
			void robot_move(const OptVec & ctr);

			ObjLogger ol;
			TimeMeasurement tm;

			const State state_estimation_t(const double t);

		public:
			RobotFollower(const int _id, const FSpec & _fs, const int _n, const double dt, FollowerDefect d,
					const Vector & initial_X, const Vector & initial_V, const Vector & initial_OMEGA, const Matrix & initial_R, const char * filename);

			const Vector get_position_t(const double t);
			const Vector get_position();
			const void rotate(double ag, double newp, double newq);
			const FSpec fspec() const { return fs; }

			const double p() const { return fs.p; }
			const double q() const { return fs.q; }
			const double r() const { return fs.r; }

			void run(const Trajectory leadertraj_);
			std::vector<double> get_rots(OptVec * traj, const int nsamples);
			Samples sample_trajectory(OptVec * traj, const int len, const int nsamples, const int msamples, const int es, const char * filename);

			void write_results() { robot_dynamics.write_to_mfile(); }
			const FSpec get_fsorig(){ return fsoriginal;}

			void post_run();
		};
	}
}
#endif
