#ifndef ROBOT_LEADER_H
#define ROBOT_LEADER_H

#include "robot.h"

#include "../measurement/obj_logger.h"
#include "../measurement/time_measurement.h"

typedef std::vector<double> Distance;
typedef imr::logger::ObjLogger ObjLogger;
typedef imr::time::TimeMeasurement TimeMeasurement;

namespace imr {
	namespace robot {
		class RobotLeader: public Robot
		{
		private:
			void adapt_limitations();
			void reorganize();
			void init();
			void robot_move(const OptVec & ctr);
		
			Trajectory ctr_history;
			Distance ctr_dist;
			double traveled_s_fdif;
			double diameter;
			double rotations;
			ObjLogger ol;
			TimeMeasurement tm;

		public:
			RobotLeader(const int _id, const int _n, const Vector & initial_X, const Vector & initial_V, const Vector & initial_OMEGA,
			    	   const Matrix & initial_R, const char * filename);

			const FDiff get_formation_difference(Samples smp, FSpec fs, FSpec fs_original, std::vector<double> rots, int vypis);
			const int traj_size() const;
			const Trajectory get_traj();
			Trajectory init_ctr(const int n, const double p, const double q);
			double distance(double x1, double y1, double x2, double y2);
			void run();
			int check_trajectory(OptVec * traj, const int len);
			Samples sample_trajectory(OptVec * traj, const int len, const int nsamples, const int msamples, const int es, const char * filename);
		};
	}
}
#endif
