#ifndef _SIMULATION_SETTINGS_H_
#define _SIMULATION_SETTINGS_H_

namespace imr {
    namespace settings {
        /// +-------------------------------------------+
        /// | Debuging => comment to disable the output |
        /// +-------------------------------------------+
        // #define DEBUG_QUADROTOR_DYNAMICS
		// #define DEBUG_QUADROTOR_MOVE
		// #define DEBUG_DYNAMICS
		// #define DEBUG_OBSTACLE
		#define LIST_LEADERS_LIMITATIONS

    	/// +---------------+
        /// | Map constants |
        /// +---------------+
        // #define MAP_FILE "../world_maps/map3real.txt"
		#define MAP_FILE "../world_maps/map0.txt"

        /// +----------------------------------------+
        /// | Simulation and robot dynamics settings |
        /// +----------------------------------------+
        // Sample time - used for integrals in quadrotor.cpp
        static const double DELTA_T = 0.0005; // [s]
        // Turn time - used to turn the quadrotor in quadrotor.cpp
        static const double TURN_T = 0.05; // [s]
        // Maximal allowed tangential velocity difference
        static const double V_TOL_XY = 0.05; // [m/s]
        static const double V_TOL_Z = 0.05; // [m/s]
        // Maximal allowed angle difference
        static const double A_TOL = 3.14159265358979323846/3600;

        // Dynamic limitations
 	static const double AG_MIN        = -3.14159265358979323846;  // [rad] //> -3.14159265358979323846
        static const double AG_MAX        =  3.14159265358979323846;  // [rad] //>  3.14159265358979323846
        static const double VT_MIN        =  0.0;  // [m/s]   //>  0.0
        static const double VT_MAX        =  0.6;  // [m/s]   //>  0.6
        static const double VN_MIN        = -0.3;  // [m/s]   //> -0.3
        static const double VN_MAX        =  0.1;  // [m/s]   //>  0.1
        static const double CURV_MIN      = -1.0;  // [1/m]   //> -1.0
        static const double CURV_MAX      =  1.0;  // [1/m]   //>  1.0
        static const double AT_MIN        = -1.0;  // [m/s^2] //> -1.0
        static const double AT_MAX        =  1.0;  // [m/s^2] //>  1.0
        static const double AN_MIN        = -1.5;  // [m/s^2] //> -1.5
        static const double AN_MAX        =  1.0;  // [m/s^2] //>  1.0

        // Control period
        static const double TIME_EXECUTE = 0.2; // [s]

        // Solver bounds
        static const double LEADER_TIME_STEP_MIN =  0.2; // [s]
        static const double LEADER_TIME_STEP_MAX = 10.0; // [s]

        // Leader's constants
        static const int LEADER_N_SAMPLES = 10;
        static const int LEADER_M_SAMPLES = 50;
        // static const double DISTANCE_TO_GOAL_S = 0.2;
        static const double DISTANCE_TO_GOAL = 0.4; // [m]

        static const double LEADER_OBJ_TOF =  50.0;		// Time of flight
        static const double LEADER_OBJ_VDI =   0.25;		// Velocity difference
        static const double LEADER_OBJ_CDI =   0.25;		// Curvature difference
        static const double LEADER_OBJ_CPE =   0.25;		// Curvature penalty
        static const double LEADER_OBJ_OBS = 150.0;		// Obstacle avoidance
        static const double LEADER_OBJ_D2G = 100.0;		// Distance to goal
        static const double LEADER_OBJ_TDI =  75.0;		// Traveled distance	
        static const double LEADER_OBJ_TURN = 75.0;		// Rotation maneuver penalty 

        static const double P_TRESHOLD = 0.015;			// [m] -> Treshold for followers p regulation
        static const double Q_TRESHOLD = 0.015;			// [m] -> Treshold for followers q regulation
        static const double R_TRESHOLD = 0.015;			// [m] -> Treshold for followers r regulation
        										// RobotLeader::get_formation_difference()

        // Follower's constants
        static const int FOLLOWER_N_SAMPLES = 10;

        static const double FOLLOWER_CONSTR_EUC = 1.5;		// Maximal allowed distance from
        											// desired position in formation

        static const double FOLLOWER_OBJ_PS  =  25.0;		// P parameter - sum of differences     //>  25
        static const double FOLLOWER_OBJ_PM  =  80.0;		// P parameter - maximal difference     //>  75
        static const double FOLLOWER_OBJ_QS  =  25.0;		// Q parameter - sum of differences     //>  25
        static const double FOLLOWER_OBJ_QM  =  80.0;		// Q parameter - maximal difference     //>  75
        static const double FOLLOWER_OBJ_RS  =   1.0;		// R parameter - sum of differences     //>   5
        static const double FOLLOWER_OBJ_RM  =  30.0;		// R parameter - maximal difference     //>  25
        static const double FOLLOWER_OBJ_VDI =   3.0;		// Velocity difference                  //>   0
        static const double FOLLOWER_OBJ_CDI =   3.0;		// Curvature difference					//>   0
        static const double FOLLOWER_OBJ_OBS =  100.0;		// Obstacle collision avoidance         //> 100
        static const double FOLLOWER_OBJ_FOL =   0.0; //75		// Formation member collision avoidance //>   0

        /// +---------------------------+
        /// | Logging of the simulation |
        /// +---------------------------+
        // Sample time - used for exporting data to matlab
        // (int)(DELTA_T/SAMPLE_T) is used
        static const double SAMPLE_T = 0.005; // [s]
        // Comment to disable control logging
        #define LOG_CONTROL
    }
}
#endif
