 /**
  * CFSQP solver
  * @author Zdenek Kasl (kaslzden@fel.cvut.cz)
  * 
  * Based on CTU FEE solver wrapper by Jan Svab (7.4.2010).
  */

#ifndef SOLVER_H
#define SOLVER_H

#include <cstdlib>
#include <iostream>

#include "../types.h"
#include "../measurement/obj_logger.h"
#include "../measurement/time_measurement.h"
typedef imr::logger::ObjLogger ObjLogger;
typedef imr::time::TimeMeasurement TimeMeasurement;

class Solver {
private:
	const Vector start_point;			//!< Current position

	Trajectory trajectory;		//!< Robot control points i.e. the solution
	const int ctr_len;					//!< Control points count
	const int id;						//!< ID of the robot (0 - leader; 1 to n+1 - index of the follower)
	const int n;						//!< Number of constant duration intervals
	const OptVec lb;					//!< Lower bounds
	const OptVec ub;					//!< Upper bounds
	const double ra;					//!< Critical radius
	const double rs;					//!< Safety radius
	const FSpec fs;					//!< Formation description
	TimeMeasurement tm;
	int status;

	friend void obj(int nparam, int j, double *x, double *fj, void *cd);
	friend void constr(int nparam, int j, double *x, double *gj, void *cd);
	friend const int call_solver(Solver *co);

public:
	Solver( const Vector & _sp,
			const int _ctr_len,
			const int _id,
			const int _n,
			const Trajectory & traj,
			const OptVec _lb,
			const OptVec _ub,
			const double _ra,
			const double _rs,
			const FSpec & _fs );

	~Solver() {}

	const Trajectory get_solution() const { return trajectory; }
	double get_fitness();
	const int get_status() const { return status; }

};
#endif
